CREATE OR REPLACE PACKAGE BODY
  -- ------------------------------------------------------------------------
  --     moto_evdo_load_pkgb.sql
  --     Copyright (c) 2005, WatchMark Corp.
  -- ------------------------------------------------------------------------
  Moto_Evdo_Load
   IS
	g_ent_name	VARCHAR2(30);
	g_ent_tab	VARCHAR2(30);

-----------------------------------------------------------------------
---------------------------  TRANS_TO_TAB  ----------------------------
-----------------------------------------------------------------------
 -- Given a table name, this function will translate it to its corresponding traffic table name
 -- by stripping the suffix (ie the last 4 characters) and appends '_TAB' to it

   FUNCTION trans_to_tab(p_src_table_name VARCHAR2) RETURN VARCHAR2
   IS
   BEGIN

	RETURN SUBSTR(p_src_table_name, 1, LENGTH(p_src_table_name)-4)||'_TAB';

   END trans_to_tab;

-----------------------------------------------------------------------
---------------------------  TRANS_TO_LTM  ----------------------------
-----------------------------------------------------------------------
-- Given a table name, this function will translate it to its corresponding ltm table name
-- by stripping the suffix (ie the last 4 characters) and appends '_LTM' to it

   FUNCTION trans_to_ltm(p_src_table_name VARCHAR2) RETURN VARCHAR2
   IS
   BEGIN
	RETURN SUBSTR(p_src_table_name, 1, LENGTH(p_src_table_name)-4)||'_LTM';
   END trans_to_ltm;

-----------------------------------------------------------------------
-------------------------  GET_ENTITY_INFO  ---------------------------
-----------------------------------------------------------------------

   PROCEDURE get_entity_info ( pSrcTable  IN  VARCHAR2 )
   IS
	v_traffic_tab	VARCHAR2(30);
   BEGIN

	v_traffic_tab := trans_to_tab(pSrcTable);
	SELECT UPPER(ENTITY_TABLE), UPPER(ENTITY_NAME) INTO g_ent_tab, g_ent_name FROM WMN_ENTITY WHERE ENTITY_ID IN
	(SELECT ENTITY_ID FROM WMD_DICT_TABLE WHERE table_name=v_traffic_tab);

   END get_entity_info;

-----------------------------------------------------------------------
-----------------------  CAGENUMBER_ENT_UPDATE  -----------------------
-----------------------------------------------------------------------

   PROCEDURE cagenumber_ent_update (pSrcTable IN VARCHAR2)
   IS
	cInstanceId	INTEGER;
	sqlStr       	VARCHAR2(4000);
	selectStr       VARCHAR2(2000);
	cnodekind	VARCHAR2(256);
	cCageNumber	INTEGER;
	v_ent_tab	VARCHAR2(30) := 'MOT_ENT_BSCDO_CARD';
	v_wml_tab	VARCHAR2(30) := 'WML_BSC_DO_CARD_MAP';
	v_done		BOOLEAN := FALSE;
   BEGIN

	sqlStr := 'MERGE INTO ' || v_ent_tab || ' D USING ('
			|| ' SELECT wml.instance_id, MAX(tmp.nodekind) AS nodekind, MAX(SUBSTR(tmp.physicaldeviceid, INSTR(tmp.physicaldeviceid, ''-'', 1, 2)+1,'
			|| ' INSTR( tmp.physicaldeviceid, ''-'', 1, 3) - INSTR( tmp.physicaldeviceid, ''-'', 1, 2)-1)) AS cage_number'
			|| ' FROM ' || v_ent_tab || ' ce, ' || v_wml_tab || ' wml, ' || pSrcTable || ' tmp, ' || g_ent_tab || ' ent'
			|| ' WHERE wml.instance_id = ce.instance_id AND wml.loaderkey = '
			|| ' SUBSTR(tmp.ID, 1, INSTR(tmp.ID, ''-'', 1, 1)-1) || ''-'' ||'
			|| ' SUBSTR(tmp.ID, INSTR(tmp.ID, ''-'', 1, 1)+1, '
			|| ' INSTR(tmp.ID, ''-'', 1, 2) - INSTR(tmp.ID, ''-'', 1, 1)-1)'
			|| ' || ''-'' || ''' || g_ent_name || ''' || '
			|| ' SUBSTR(tmp.ID, INSTR(tmp.ID, ''-'', 1, 2)+1, LENGTH(tmp.ID))'
			|| ' GROUP BY wml.instance_id'
			|| ' ) S'
			|| ' ON ( D.instance_id = S.instance_id ) WHEN MATCHED THEN'
			|| ' UPDATE SET D.cage_number = DECODE(S.nodekind, ''IP-BSC-DO'', S.cage_number, 1)'
			|| ' WHEN NOT MATCHED THEN'
			|| ' INSERT (D.instance_id, D.cage_number) VALUES (S.instance_id, S.cage_number)';

	Wml_Load.repeat_on_dup_val(sqlStr);

   END cagenumber_ent_update;

-----------------------------------------------------------------------
-----------------------  CARDKIND_ENT_UPDATE  -------------------------
-----------------------------------------------------------------------

   PROCEDURE cardkind_ent_update ( pSrcTable IN VARCHAR2 )
   IS
	sqlStr	VARCHAR2(4000);
   BEGIN

	sqlStr := 'MERGE INTO ' || g_ent_tab || ' D USING ('
			|| ' SELECT tmp.instance_id, MAX(tmp.cardkind) AS cardkind FROM ' || pSrcTable || ' tmp'
			|| ' GROUP BY tmp.instance_id ) S'
			|| ' ON ( D.instance_id = S.instance_id ) WHEN MATCHED THEN'
			|| ' UPDATE SET D.cardkind = S.cardkind'
			|| ' WHEN NOT MATCHED THEN'
			|| ' INSERT (D.instance_id, D.cardkind) VALUES (S.instance_id, S.cardkind)';

	-- Protect against parallel loading
	Wml_Load.repeat_on_dup_val(sqlStr);

   END cardkind_ent_update;

-----------------------------------------------------------------------
------------------------  NODEKIND_ENT_UPDATE  ------------------------
-----------------------------------------------------------------------
-- This procedure also does the physical device id update as well

   PROCEDURE nodekind_ent_update ( pSrcTable IN VARCHAR2 )
   IS
	sqlStr       	VARCHAR2(4000);
   BEGIN

	 -- There's no physicaldeviceid in BSC_DO source table
	 IF g_ent_name = 'BSC_DO' THEN

	    sqlStr := 'MERGE INTO ' || g_ent_tab || ' D USING ('
			|| ' SELECT tmp.instance_id, MAX(tmp.nodekind) AS nodekind FROM ' || pSrcTable || ' tmp'
			|| ' GROUP BY tmp.instance_id ) S'
			|| ' ON ( D.instance_id = S.instance_id ) WHEN MATCHED THEN'
			|| ' UPDATE SET D.nodekind = S.nodekind'
			|| ' WHEN NOT MATCHED THEN'
			|| ' INSERT (D.instance_id, D.nodekind) VALUES (S.instance_id, S.nodekind)';

	    -- Protect against parallel loading
   	    Wml_Load.repeat_on_dup_val(sqlStr);

	ELSE

	    sqlStr := 'MERGE INTO ' || g_ent_tab || ' D USING ('
			|| ' SELECT tmp.instance_id, MAX(tmp.nodekind) AS nodekind, MAX(tmp.physicaldeviceid) AS physicaldeviceid FROM ' || pSrcTable || ' tmp'
			|| ' GROUP BY tmp.instance_id ) S'
			|| ' ON ( D.instance_id = S.instance_id ) WHEN MATCHED THEN'
			|| ' UPDATE SET D.nodekind = S.nodekind, D.physicaldeviceid = S.physicaldeviceid'
			|| ' WHEN NOT MATCHED THEN'
			|| ' INSERT (D.instance_id, D.nodekind) VALUES (S.instance_id, S.nodekind)';

	    -- Protect against parallel loading
   	    Wml_Load.repeat_on_dup_val(sqlStr);

	END IF;

   END nodekind_ent_update;

-----------------------------------------------------------------------
---------------------------  get_range  -------------------------------
-----------------------------------------------------------------------
-- Supporting procedure for update_wml_bsc_do_card_map

   PROCEDURE get_range(p_table_name IN VARCHAR2, p_mindate OUT VARCHAR2, p_maxdate OUT VARCHAR2, p_minkey OUT VARCHAR2, p_maxkey OUT VARCHAR2, p_input_row_count OUT INTEGER)
   AS
	sqlStr	VARCHAR2(4000);
   BEGIN

	sqlStr := 'SELECT TO_CHAR(MIN(tstamp),''MM/DD/YYYY HH24:MI''),'
		|| ' TO_CHAR(MAX(tstamp),''MM/DD/YYYY HH24:MI''),'
		|| ' MIN( SUBSTR(ID, 1, INSTR(ID, ''-'', 1, 1)-1) || ''-'' ||'
		|| ' SUBSTR(ID, INSTR(ID, ''-'', 1, 1)+1, INSTR(ID, ''-'', 1, 2)-INSTR(ID, ''-'', 1, 1)-1) || ''-'' || '''
		|| g_ent_name || ''' || SUBSTR(ID, INSTR(ID, ''-'', 1, 2)+1, LENGTH(ID)) ),'
		|| ' MAX( SUBSTR(ID, 1, INSTR(ID, ''-'', 1, 1)-1) || ''-'' ||'
		|| ' SUBSTR(ID, INSTR(ID, ''-'', 1, 1)+1, INSTR(ID, ''-'', 1, 2)-INSTR(ID, ''-'', 1, 1)-1) || ''-'' ||'''
		|| g_ent_name || ''' || SUBSTR(ID, INSTR(ID, ''-'', 1, 2)+1, LENGTH(ID)) ),'
		|| ' COUNT(*)'
		|| ' FROM ' || p_table_name;

	EXECUTE IMMEDIATE sqlStr INTO p_mindate, p_maxdate, p_minkey, p_maxkey, p_input_row_count;

   END get_range;

-----------------------------------------------------------------------
---------------------  update_wml_bsc_do_card_map  --------------------
-----------------------------------------------------------------------
-- Populates the WML mapping table with the corresponding instance_id for a given fullkey
-- This is suppose to improve performance by reducing the Find_Instance calls

   PROCEDURE update_wml_bsc_do_card_map (p_tmp_tabname IN VARCHAR2, p_minkey IN VARCHAR2, p_maxkey IN VARCHAR2)
   AS
	sqlStr		VARCHAR2(4000);
	v_wml_map_tab	VARCHAR2(30) := 'WML_BSC_DO_CARD_MAP';
	v_done		BOOLEAN := FALSE;
   BEGIN

	sqlStr := 'INSERT INTO ' || v_wml_map_tab || ' (loaderkey, SYSTEM, bsc_do, bsc_do_card, instance_id)'
    		|| ' SELECT tmptab.loaderkey AS loaderkey,'
		|| ' tmptab.system_id, tmptab.bsc_do_id, tmptab.bsc_do_card_id,'
		|| ' Wm_Generic_Find_Instance.Find_Instance('
		|| ' tmptab.system_id || ''-'' || tmptab.bsc_do_id || ''-'' || tmptab.bsc_do_card_id , ''BSC_DO_Card'') AS instance_id'
      		|| ' FROM ('
		|| '		 SELECT /*+ CARDINALITY(tmp, 1000) */'
		|| '		 DISTINCT('
		|| '		 SUBSTR(tmp.ID, 1, INSTR(tmp.ID, ''-'', 1, 1)-1) || ''-'' || '
		|| '		 SUBSTR(ID, INSTR(ID, ''-'', 1, 1)+1, INSTR(ID, ''-'', 1, 2)-INSTR(ID, ''-'', 1, 1)-1) || ''-'' || '''
		|| g_ent_name || ''' || SUBSTR(ID, INSTR(ID, ''-'', 1, 2)+1, LENGTH(ID))'
		|| '		 ) AS loaderkey,'
		|| '		 SUBSTR(tmp.ID, 1, INSTR(tmp.ID, ''-'', 1, 1)-1) AS system_id,'
		|| '		 SUBSTR(ID, INSTR(ID, ''-'', 1, 1)+1, INSTR(ID, ''-'', 1, 2)-INSTR(ID, ''-'', 1, 1)-1) AS bsc_do_id,'
		|| '		 '''|| g_ent_name || ''' || SUBSTR(ID, INSTR(ID, ''-'', 1, 2)+1, LENGTH(ID)) AS bsc_do_card_id'
		|| ' 	 FROM ' || p_tmp_tabname || ' tmp'
  		|| ' ) tmptab,'
		|| v_wml_map_tab || ' wml'
		|| '	WHERE wml.loaderkey (+)= tmptab.loaderkey'
		|| '	AND wml.loaderkey(+) BETWEEN :v_minkey AND :v_maxkey'
		|| '	AND wml.loaderkey IS NULL';

	WHILE (NOT v_done)
	LOOP
	   BEGIN
		EXECUTE IMMEDIATE sqlStr USING p_minkey, p_maxkey;
		COMMIT;
		v_done := TRUE;
	   EXCEPTION
		WHEN DUP_VAL_ON_INDEX THEN NULL;
	   END;
	END LOOP;

END update_wml_bsc_do_card_map;

-----------------------------------------------------------------------
------------------------ POPULATE_BSCCARD_CC  -------------------------
-----------------------------------------------------------------------

   PROCEDURE POPULATE_BSCCARD_CC   ( pSrcTable IN VARCHAR2, ptstamp   IN  VARCHAR2 )
   IS
	sqlStr			VARCHAR2(32000);
	v_system_id_str		VARCHAR2(500);
	v_bscdo_id_str		VARCHAR2(500);
	v_bscdo_card_id_str	VARCHAR2(2500);
	v_fullkey_cols_str	VARCHAR2(5000);
	v_mindate 		VARCHAR2(20);
	v_maxdate 		VARCHAR2(20);
	v_minkey 		VARCHAR2(256);
	v_maxkey 		VARCHAR2(256);
	v_input_row_count 	INTEGER := 0; -- useful for CARDINALITY hints should we need them
	v_traffic_ltm		VARCHAR2(30);
	v_wml_map_tab 		VARCHAR2(30) := 'WML_BSC_DO_CARD_MAP';
	v_done			BOOLEAN := FALSE;
   BEGIN

	v_traffic_ltm := trans_to_ltm(pSrcTable);
	get_range( pSrcTable, v_mindate, v_maxdate, v_minkey, v_maxkey, v_input_row_count);

	v_system_id_str := 'SUBSTR(ltm.ID, 1, INSTR(ltm.ID, ''-'', 1, 1)-1)';
	v_bscdo_id_str := 'SUBSTR(ltm.ID, INSTR(ltm.ID, ''-'', 1, 1)+1, INSTR(ltm.ID, ''-'', 1, 2)-INSTR(ltm.ID, ''-'', 1, 1)-1)';
	v_bscdo_card_id_str := '''' || g_ent_name || ''' || SUBSTR(ltm.ID, INSTR(ltm.ID, ''-'', 1, 2)+1, LENGTH(ltm.ID))';

	v_fullkey_cols_str := v_system_id_str || ' || ''-'' || ' || v_bscdo_id_str || ' || ''-'' || ' || v_bscdo_card_id_str;

	update_wml_bsc_do_card_map( pSrcTable, v_minkey, v_maxkey);

	sqlStr := ' MERGE /*+ USE_HASH(d) INDEX(d) */ INTO mot_tf_do_pm_bsccard_tab D'
			|| ' USING ( SELECT /*+ ORDERED */ DISTINCT('
			|| v_fullkey_cols_str || ') AS loaderkey, wml.instance_id instance_id,'
			|| ' tstamp, blockingtimeofcc, collectionperiod, cpuuserate_avg, cpuuserate_max, memoryuserate_avg, memoryuserate_max, suspectflag'
			|| ' FROM ' || v_traffic_ltm || ' ltm, ' || v_wml_map_tab || ' wml'
			|| ' WHERE wml.system = ' || v_system_id_str || ' AND'
			|| ' wml.bsc_do = ' || v_bscdo_id_str || ' AND'
			|| ' wml.bsc_do_card = ' || v_bscdo_card_id_str || ' AND'
			|| ' wml.loaderkey = ' || v_fullkey_cols_str
			|| ' ) S'
			|| ' ON ( D.id = S.loaderkey  '
			|| ' AND D.tstamp = S.tstamp '
			|| ' AND D.tstamp BETWEEN TO_DATE(:mindate,''MM/DD/YYYY HH24:MI'') AND TO_DATE(:maxdate,''MM/DD/YYYY HH24:MI'') '
			|| ' AND D.id BETWEEN :minkey AND :maxkey)'
			|| ' WHEN MATCHED THEN '
			|| ' UPDATE SET  '
			|| ' D.instance_id = S.instance_id,'
			|| ' D.blockingtimeofbscdocard = S.blockingtimeofcc,'
			|| ' D.collectionperiod = S.collectionperiod,'
			|| ' D.cpuuserate_avg = S.cpuuserate_avg,'
			|| ' D.cpuuserate_max = S.cpuuserate_max,'
			|| ' D.memoryuserate_avg = S.memoryuserate_avg,'
			|| ' D.memoryuserate_max = S.memoryuserate_max,'
			|| ' D.suspectflag = S.suspectflag'
			|| ' WHEN NOT MATCHED THEN '
			|| ' INSERT ( '
			|| ' D.ID, D.tstamp, D.instance_id, D.blockingtimeofbscdocard,'
			|| '  D.collectionperiod, D.cpuuserate_avg, D.cpuuserate_max,'
			|| '  D.memoryuserate_avg, D.memoryuserate_max, D.suspectflag'
			|| ' ) VALUES ( '
			|| ' S.loaderkey,'
			|| ' S.tstamp,'
			|| ' S.instance_id,'
			|| ' S.blockingtimeofcc,'
			|| ' S.collectionperiod,'
			|| ' S.cpuuserate_avg,'
			|| ' S.cpuuserate_max,'
			|| ' S.memoryuserate_avg,'
			|| ' S.memoryuserate_max,'
			|| ' S.suspectflag'
			|| ' )';

	WHILE (NOT v_done)
	LOOP
	   BEGIN
		EXECUTE IMMEDIATE sqlStr USING v_mindate, v_maxdate, v_minkey, v_maxkey;
		COMMIT;
		v_done := TRUE;
	   EXCEPTION
		WHEN DUP_VAL_ON_INDEX THEN NULL;
	   END;
	END LOOP;

   END POPULATE_BSCCARD_CC;

-----------------------------------------------------------------------
------------------------ POPULATE_BSCCARD_TC  -------------------------
-----------------------------------------------------------------------

   PROCEDURE POPULATE_BSCCARD_TC   ( pSrcTable IN VARCHAR2, ptstamp IN VARCHAR2 )
   IS
	sqlStr			VARCHAR2(32000);
	v_system_id_str		VARCHAR2(500);
	v_bscdo_id_str		VARCHAR2(500);
	v_bscdo_card_id_str	VARCHAR2(2500);
	v_fullkey_cols_str	VARCHAR2(5000);
	v_mindate 		VARCHAR2(20);
	v_maxdate 		VARCHAR2(20);
	v_minkey 		VARCHAR2(256);
	v_maxkey 		VARCHAR2(256);
	v_input_row_count 	INTEGER := 0; -- useful for CARDINALITY hints should we need them
	v_traffic_ltm		VARCHAR2(30);
	v_wml_map_tab 		VARCHAR2(30) := 'WML_BSC_DO_CARD_MAP';
	v_done			BOOLEAN := FALSE;
   BEGIN

	v_traffic_ltm := trans_to_ltm(pSrcTable);
	get_range( pSrcTable, v_mindate, v_maxdate, v_minkey, v_maxkey, v_input_row_count);

	v_system_id_str := 'SUBSTR(ltm.ID, 1, INSTR(ltm.ID, ''-'', 1, 1)-1)';
	v_bscdo_id_str := 'SUBSTR(ltm.ID, INSTR(ltm.ID, ''-'', 1, 1)+1, INSTR(ltm.ID, ''-'', 1, 2)-INSTR(ltm.ID, ''-'', 1, 1)-1)';
	v_bscdo_card_id_str := '''' || g_ent_name || ''' || SUBSTR(ltm.ID, INSTR(ltm.ID, ''-'', 1, 2)+1, LENGTH(ltm.ID))';

	v_fullkey_cols_str := v_system_id_str || ' || ''-'' || ' || v_bscdo_id_str || ' || ''-'' || ' || v_bscdo_card_id_str;

	update_wml_bsc_do_card_map( pSrcTable, v_minkey, v_maxkey);

	sqlStr := ' MERGE /*+ USE_HASH(d) INDEX(d) */ INTO mot_tf_do_pm_bsccard_tab D'
			|| ' USING ( SELECT /*+ ORDERED */ DISTINCT('
			|| v_fullkey_cols_str || ') AS loaderkey, wml.instance_id instance_id,'
			|| ' tstamp, blockingtimeoftc, bufferuserateoftc_avg, bufferuserateoftc_max, collectionperiod,'
			|| ' cpuuserate_avg, cpuuserate_max,memoryuserate_avg,memoryuserate_max, suspectflag'
			|| ' FROM ' || v_traffic_ltm || ' ltm, ' || v_wml_map_tab || ' wml'
			|| ' WHERE wml.system = ' || v_system_id_str || ' AND'
			|| ' wml.bsc_do = ' || v_bscdo_id_str || ' AND'
			|| ' wml.bsc_do_card = ' || v_bscdo_card_id_str || ' AND'
			|| ' wml.loaderkey = ' || v_fullkey_cols_str
			|| ' ) S'
			|| ' ON ( D.id = S.loaderkey  '
			|| ' AND D.tstamp = S.tstamp '
			|| ' AND D.tstamp BETWEEN TO_DATE(:mindate,''MM/DD/YYYY HH24:MI'') AND TO_DATE(:maxdate,''MM/DD/YYYY HH24:MI'') '
			|| ' AND D.id BETWEEN :minkey AND :maxkey)'
			|| ' WHEN MATCHED THEN '
			|| ' UPDATE SET  '
			|| ' D.instance_id = S.instance_id,'
			|| ' D.blockingtimeofbscdocard = S.blockingtimeoftc,'
			|| ' D.bufferuserateoftc_avg = S.bufferuserateoftc_avg,'
			|| ' D.bufferuserateoftc_max = S.bufferuserateoftc_max,'
			|| ' D.collectionperiod = S.collectionperiod,'
			|| ' D.cpuuserate_avg = S.cpuuserate_avg,'
			|| ' D.cpuuserate_max = S.cpuuserate_max,'
			|| ' D.memoryuserate_avg = S.memoryuserate_avg,'
			|| ' D.memoryuserate_max = S.memoryuserate_max,'
			|| ' D.suspectflag = S.suspectflag'
			|| ' WHEN NOT MATCHED THEN '
			|| ' INSERT ( '
			|| ' D.ID, D.tstamp, D.instance_id, D.blockingtimeofbscdocard, D.bufferuserateoftc_avg,'
			|| ' D.bufferuserateoftc_max, D.collectionperiod, D.cpuuserate_avg, D.cpuuserate_max,'
			|| '  D.memoryuserate_avg, D.memoryuserate_max, D.suspectflag'
			|| ' ) VALUES ( '
			|| ' S.loaderkey,'
			|| ' S.tstamp,'
			|| ' S.instance_id,'
			|| ' S.blockingtimeoftc,'
			|| ' S.bufferuserateoftc_avg,'
			|| ' S.bufferuserateoftc_max,'
			|| ' S.collectionperiod,'
			|| ' S.cpuuserate_avg,'
			|| ' S.cpuuserate_max,'
			|| ' S.memoryuserate_avg,'
			|| ' S.memoryuserate_max,'
			|| ' S.suspectflag'
			|| ' )';

	WHILE (NOT v_done)
	LOOP
	   BEGIN
		EXECUTE IMMEDIATE sqlStr USING v_mindate, v_maxdate, v_minkey, v_maxkey;
		COMMIT;
		v_done := TRUE;
	   EXCEPTION
		WHEN DUP_VAL_ON_INDEX THEN NULL;
	   END;
	END LOOP;

   END POPULATE_BSCCARD_TC;

-----------------------------------------------------------------------
------------------------ POPULATE_BSCCARD_TRA -------------------------
-----------------------------------------------------------------------

   PROCEDURE POPULATE_BSCCARD_TRA   ( pSrcTable IN VARCHAR2, ptstamp   IN  VARCHAR2 )
   IS
	sqlStr			VARCHAR2(32000);
	v_system_id_str		VARCHAR2(500);
	v_bscdo_id_str		VARCHAR2(500);
	v_bscdo_card_id_str	VARCHAR2(2500);
	v_fullkey_cols_str	VARCHAR2(5000);
	v_mindate 		VARCHAR2(20);
	v_maxdate 		VARCHAR2(20);
	v_minkey 		VARCHAR2(256);
	v_maxkey 		VARCHAR2(256);
	v_input_row_count 	INTEGER := 0; -- useful for CARDINALITY hints should we need them
	v_traffic_ltm		VARCHAR2(30);
	v_wml_map_tab 		VARCHAR2(30) := 'WML_BSC_DO_CARD_MAP';
	v_done			BOOLEAN := FALSE;
   BEGIN

	v_traffic_ltm := trans_to_ltm(pSrcTable);
	get_range( pSrcTable, v_mindate, v_maxdate, v_minkey, v_maxkey, v_input_row_count);

	v_system_id_str := 'SUBSTR(ltm.ID, 1, INSTR(ltm.ID, ''-'', 1, 1)-1)';
	v_bscdo_id_str := 'SUBSTR(ltm.ID, INSTR(ltm.ID, ''-'', 1, 1)+1, INSTR(ltm.ID, ''-'', 1, 2)-INSTR(ltm.ID, ''-'', 1, 1)-1)';
	v_bscdo_card_id_str := '''' || g_ent_name || ''' || SUBSTR(ltm.ID, INSTR(ltm.ID, ''-'', 1, 2)+1, LENGTH(ltm.ID))';

	v_fullkey_cols_str := v_system_id_str || ' || ''-'' || ' || v_bscdo_id_str || ' || ''-'' || ' || v_bscdo_card_id_str;

	update_wml_bsc_do_card_map( pSrcTable, v_minkey, v_maxkey);

	sqlStr := ' MERGE /*+ USE_HASH(d) INDEX(d) */ INTO mot_tf_do_pm_bsccard_tab D'
			|| ' USING ( SELECT /*+ ORDERED */ DISTINCT('
			|| v_fullkey_cols_str || ') AS loaderkey, wml.instance_id instance_id,'
			|| ' tstamp, BLOCKINGTIMEOFTRACARD, collectionperiod, cpuuserate_avg, cpuuserate_max,'
			|| ' memoryuserate_avg, memoryuserate_max, suspectflag'
			|| ' FROM ' || v_traffic_ltm || ' ltm, ' || v_wml_map_tab || ' wml'
			|| ' WHERE wml.system = ' || v_system_id_str || ' AND'
			|| ' wml.bsc_do = ' || v_bscdo_id_str || ' AND'
			|| ' wml.bsc_do_card = ' || v_bscdo_card_id_str || ' AND'
			|| ' wml.loaderkey = ' || v_fullkey_cols_str
			|| ' ) S'
			|| ' ON ( D.id = S.loaderkey  '
			|| ' AND D.tstamp = S.tstamp '
			|| ' AND D.tstamp BETWEEN TO_DATE(:mindate,''MM/DD/YYYY HH24:MI'') AND TO_DATE(:maxdate,''MM/DD/YYYY HH24:MI'') '
			|| ' AND D.id BETWEEN :minkey AND :maxkey)'
      			|| ' WHEN MATCHED THEN '
			|| ' UPDATE SET  '
			|| ' D.instance_id = S.instance_id,'
			|| ' D.blockingtimeofbscdocard = S.BLOCKINGTIMEOFTRACARD,'
			|| ' D.collectionperiod = S.collectionperiod,'
			|| ' D.cpuuserate_avg = S.cpuuserate_avg,'
			|| ' D.cpuuserate_max = S.cpuuserate_max,'
			|| ' D.memoryuserate_avg = S.memoryuserate_avg,'
			|| ' D.memoryuserate_max = S.memoryuserate_max,'
			|| ' D.suspectflag = S.suspectflag'
			|| ' WHEN NOT MATCHED THEN '
			|| ' INSERT ( '
			|| ' D.ID, D.tstamp, D.instance_id, D.blockingtimeofbscdocard,'
			|| '  D.collectionperiod, D.cpuuserate_avg, D.cpuuserate_max,'
			|| '  D.memoryuserate_avg, D.memoryuserate_max, D.suspectflag'
			|| ' ) VALUES ( '
			|| ' S.loaderkey,'
			|| ' S.tstamp,'
			|| ' S.instance_id,'
			|| ' S.BLOCKINGTIMEOFTRACARD,'
			|| ' S.collectionperiod,'
			|| ' S.cpuuserate_avg,'
			|| ' S.cpuuserate_max,'
			|| ' S.memoryuserate_avg,'
			|| ' S.memoryuserate_max,'
			|| ' S.suspectflag'
			|| ' )';

	WHILE (NOT v_done)
	LOOP
	   BEGIN
		EXECUTE IMMEDIATE sqlStr USING v_mindate, v_maxdate, v_minkey, v_maxkey;
		COMMIT;
		v_done := TRUE;
	   EXCEPTION
		WHEN DUP_VAL_ON_INDEX THEN NULL;
	   END;
	END LOOP;

   END POPULATE_BSCCARD_TRA;

-----------------------------------------------------------------------
------------------------ POPULATE_BSCCARD_SVM -------------------------
-----------------------------------------------------------------------

   PROCEDURE POPULATE_BSCCARD_SVM   ( pSrcTable IN VARCHAR2, ptstamp   IN  VARCHAR2 )
   IS
	sqlStr			VARCHAR2(32000);
	v_system_id_str		VARCHAR2(500);
	v_bscdo_id_str		VARCHAR2(500);
	v_bscdo_card_id_str	VARCHAR2(2500);
	v_fullkey_cols_str	VARCHAR2(5000);
	v_mindate 		VARCHAR2(20);
	v_maxdate 		VARCHAR2(20);
	v_minkey 		VARCHAR2(256);
	v_maxkey 		VARCHAR2(256);
	v_input_row_count 	INTEGER := 0; -- useful for CARDINALITY hints should we need them
	v_traffic_ltm		VARCHAR2(30);
	v_wml_map_tab 		VARCHAR2(30) := 'WML_BSC_DO_CARD_MAP';
	v_done			BOOLEAN := FALSE;
   BEGIN

	v_traffic_ltm := trans_to_ltm(pSrcTable);
	get_range( pSrcTable, v_mindate, v_maxdate, v_minkey, v_maxkey, v_input_row_count);

	v_system_id_str := 'SUBSTR(ltm.ID, 1, INSTR(ltm.ID, ''-'', 1, 1)-1)';
	v_bscdo_id_str := 'SUBSTR(ltm.ID, INSTR(ltm.ID, ''-'', 1, 1)+1, INSTR(ltm.ID, ''-'', 1, 2)-INSTR(ltm.ID, ''-'', 1, 1)-1)';
	v_bscdo_card_id_str := '''' || g_ent_name || ''' || SUBSTR(ltm.ID, INSTR(ltm.ID, ''-'', 1, 2)+1, LENGTH(ltm.ID))';

	v_fullkey_cols_str := v_system_id_str || ' || ''-'' || ' || v_bscdo_id_str || ' || ''-'' || ' || v_bscdo_card_id_str;

	update_wml_bsc_do_card_map( pSrcTable, v_minkey, v_maxkey);

	sqlStr := ' MERGE /*+ USE_HASH(d) INDEX(d) */ INTO mot_tf_do_pm_bsccard_tab D'
			|| ' USING ( SELECT /*+ ORDERED */ DISTINCT('
			|| v_fullkey_cols_str || ') AS loaderkey, wml.instance_id instance_id,'
			|| ' tstamp, collectionperiod, cpuuserate_avg, cpuuserate_max, memoryuserate_avg, memoryuserate_max, suspectflag'
			|| ' FROM ' || v_traffic_ltm || ' ltm, ' || v_wml_map_tab || ' wml'
			|| ' WHERE wml.system = SUBSTR(ltm.ID, 1, INSTR(ltm.ID, ''-'', 1, 1)-1) AND'
			|| ' wml.bsc_do = SUBSTR(ltm.ID, INSTR(ltm.ID, ''-'', 1, 1)+1, INSTR(ltm.ID, ''-'', 1, 2)-INSTR(ltm.ID, ''-'', 1, 1)-1) AND'
			|| ' wml.bsc_do_card = ''' || g_ent_name || ''' || SUBSTR(ltm.ID, INSTR(ltm.ID, ''-'', 1, 2)+1, LENGTH(ltm.ID)) AND'
			|| ' wml.loaderkey = loaderkey'
			|| ' ) S'
			|| ' ON ( D.id = S.loaderkey  '
			|| ' AND D.tstamp = S.tstamp '
			|| ' AND D.tstamp BETWEEN TO_DATE(:mindate,''MM/DD/YYYY HH24:MI'') AND TO_DATE(:maxdate,''MM/DD/YYYY HH24:MI'') '
			|| ' AND D.id BETWEEN :minkey AND :maxkey)'
      			|| ' WHEN MATCHED THEN '
			|| ' UPDATE SET  '
			|| ' D.instance_id = S.instance_id,'
			|| ' D.collectionperiod = S.collectionperiod,'
			|| ' D.cpuuserate_avg = S.cpuuserate_avg,'
			|| ' D.cpuuserate_max = S.cpuuserate_max,'
			|| ' D.memoryuserate_avg = S.memoryuserate_avg,'
			|| ' D.memoryuserate_max = S.memoryuserate_max,'
			|| ' D.suspectflag = S.suspectflag'
			|| ' WHEN NOT MATCHED THEN '
			|| ' INSERT ( '
			|| ' D.ID, D.tstamp, D.instance_id,'
			|| '  D.collectionperiod, D.cpuuserate_avg, D.cpuuserate_max,'
			|| '  D.memoryuserate_avg, D.memoryuserate_max, D.suspectflag'
			|| ' ) VALUES ( '
			|| ' S.loaderkey,'
			|| ' S.tstamp,'
			|| ' S.instance_id,'
			|| ' S.collectionperiod,'
			|| ' S.cpuuserate_avg,'
			|| ' S.cpuuserate_max,'
			|| ' S.memoryuserate_avg,'
			|| ' S.memoryuserate_max,'
			|| ' S.suspectflag'
			|| ' )';

	WHILE (NOT v_done)
	LOOP
	   BEGIN
		EXECUTE IMMEDIATE sqlStr USING v_mindate, v_maxdate, v_minkey, v_maxkey;
		COMMIT;
		v_done := TRUE;
	   EXCEPTION
		WHEN DUP_VAL_ON_INDEX THEN NULL;
	   END;
	END LOOP;

   END POPULATE_BSCCARD_SVM;

-----------------------------------------------------------------------
------------------------ POPULATE_BSCCARD_SCA -------------------------
-----------------------------------------------------------------------

	 PROCEDURE POPULATE_BSCCARD_SCA   ( pSrcTable IN VARCHAR2, pTSTAMP   IN  VARCHAR2 )
     IS
       sqlStr           			 VARCHAR2(32000);
	   v_system_id_str				 VARCHAR2(500);
	   v_bscdo_id_str				 VARCHAR2(500);
	   v_bscdo_card_id_str			 VARCHAR2(2500);
	   v_fullkey_cols_str			 VARCHAR2(5000);
  	   v_mindate 					 VARCHAR2(20);
  	   v_maxdate 					 VARCHAR2(20);
  	   v_minkey 					 VARCHAR2(256);
  	   v_maxkey 					 VARCHAR2(256);
  	   v_input_row_count 			 INTEGER := 0; -- useful for CARDINALITY hints should we need them
	   v_traffic_ltm				 VARCHAR2(30);
	   v_wml_map_tab 				 VARCHAR2(30) := 'WML_BSC_DO_CARD_MAP';
	 BEGIN
	 	  v_traffic_ltm := trans_to_ltm(pSrcTable);

	 	  Get_Range( pSrcTable, v_mindate, v_maxdate, v_minkey, v_maxkey, v_input_row_count);

		   v_system_id_str := 'SUBSTR(ltm.ID, 1, INSTR(ltm.ID, ''-'', 1, 1)-1)';
		   v_bscdo_id_str := 'SUBSTR(ltm.ID, INSTR(ltm.ID, ''-'', 1, 1)+1, INSTR(ltm.ID, ''-'', 1, 2)-INSTR(ltm.ID, ''-'', 1, 1)-1)';
		   v_bscdo_card_id_str := '''' || g_ent_name || ''' || SUBSTR(ltm.ID, INSTR(ltm.ID, ''-'', 1, 2)+1, LENGTH(ltm.ID))';

                   -- SEAde67967: The bscdo_card formed here is wrong hence the merge statement could not merge the data from MOT_TF_DO_PM_SCA_LTM
		   --v_bscdo_card_id_str := '''' || g_ent_name || ''' || ''_'' || SUBSTR(ent.physicaldeviceid, INSTR(ent.physicaldeviceid, ''-'', 1, 1)+1,'
		   --					   	  || 'INSTR(ent.physicaldeviceid, ''-'', 1, 2)-INSTR(ent.physicaldeviceid, ''-'', 1, 1)-1)'
		   --						  || ' || ''_'' || SUBSTR(ltm.ID, INSTR(ltm.ID, ''-'', 1, 2)+1, LENGTH(ltm.ID))';

		   v_fullkey_cols_str := v_system_id_str || ' || ''-'' || ' || v_bscdo_id_str || ' || ''-'' || ' || v_bscdo_card_id_str;

	 	   UPDATE_WML_BSC_DO_CARD_MAP( pSrcTable, v_minkey, v_maxkey);

	 	    sqlStr := ' MERGE /*+ USE_HASH(d) INDEX(d) */ INTO MOT_TF_DO_PM_BSCCARD_TAB D'
		   || ' USING ( SELECT /*+ ORDERED */ DISTINCT('
		   || v_fullkey_cols_str || ') AS loaderkey, wml.instance_id instance_id,'
		   || ' TSTAMP, CPUUSERATE_AVG, CPUUSERATE_MAX, MEMUSERATE_AVG, MEMUSERATE_MAX, SUSPECTFLAG, GRANULARITYPERIOD'
		   || ' FROM ' || v_traffic_ltm || ' ltm, ' || v_wml_map_tab || ' wml, ' || g_ent_tab || ' ent'
		   || ' WHERE wml.system = ' || v_system_id_str || ' AND'
		   || ' wml.bsc_do = ' || v_bscdo_id_str || ' AND'
		   || ' wml.bsc_do_card = ' || v_bscdo_card_id_str || ' AND'
		   || ' wml.loaderkey = ' || v_fullkey_cols_str || ' AND'
		   || ' ltm.instance_id = ent.instance_id'
		   || ' ) S'
		   || ' ON ( D.id = S.loaderkey  '
		   || ' AND D.tstamp = S.tstamp '
		   || ' AND D.tstamp BETWEEN TO_DATE(:mindate,''MM/DD/YYYY HH24:MI'') AND TO_DATE(:maxdate,''MM/DD/YYYY HH24:MI'') '
		 || ' AND D.id BETWEEN :minkey AND :maxkey)'
      	 || ' WHEN MATCHED THEN '
		 || ' UPDATE SET  '
		 || ' D.instance_id = S.instance_id,'
		 || ' D.CPUUSERATE_AVG = S.CPUUSERATE_AVG,'
		 || ' D.CPUUSERATE_MAX = S.CPUUSERATE_MAX,'
		 || ' D.MEMORYUSERATE_AVG = S.MEMUSERATE_AVG,'
		 || ' D.MEMORYUSERATE_MAX = S.MEMUSERATE_MAX,'
		 || ' D.SUSPECTFLAG = S.SUSPECTFLAG,'
		 || ' D.COLLECTIONPERIOD = S.GRANULARITYPERIOD'
		 || ' WHEN NOT MATCHED THEN '
		 || ' INSERT ( '
		 || ' D.ID, D.TSTAMP, D.INSTANCE_ID,'
		 || ' D.CPUUSERATE_AVG, D.CPUUSERATE_MAX,'
		 || ' D.MEMORYUSERATE_AVG, D.MEMORYUSERATE_MAX, D.SUSPECTFLAG, D.COLLECTIONPERIOD'
		 || ' ) VALUES ( '
		 || ' S.loaderkey,'
		 || ' S.TSTAMP,'
		 || ' S.instance_id,'
		 || ' S.CPUUSERATE_AVG,'
		 || ' S.CPUUSERATE_MAX,'
		 || ' S.MEMUSERATE_AVG,'
		 || ' S.MEMUSERATE_MAX,'
		 || ' S.SUSPECTFLAG,'
		 || ' S.GRANULARITYPERIOD'
		 || ' )';

		 EXECUTE IMMEDIATE sqlStr USING v_mindate, v_maxdate, v_minkey, v_maxkey;

	 END POPULATE_BSCCARD_SCA;

-----------------------------------------------------------------------
------------------------ POPULATE_BSCCARD_APC -------------------------
-----------------------------------------------------------------------

   PROCEDURE POPULATE_BSCCARD_APC   ( pSrcTable IN VARCHAR2, ptstamp   IN  VARCHAR2 )
   IS
	sqlStr			VARCHAR2(32000);
	v_system_id_str		VARCHAR2(500);
	v_bscdo_id_str		VARCHAR2(500);
	v_bscdo_card_id_str	VARCHAR2(2500);
	v_fullkey_cols_str	VARCHAR2(5000);
	v_mindate 		VARCHAR2(20);
	v_maxdate 		VARCHAR2(20);
	v_minkey 		VARCHAR2(256);
	v_maxkey 		VARCHAR2(256);
	v_input_row_count 	INTEGER := 0; -- useful for CARDINALITY hints should we need them
	v_traffic_ltm		VARCHAR2(30);
	v_wml_map_tab 		VARCHAR2(30) := 'WML_BSC_DO_CARD_MAP';
	v_done			BOOLEAN := FALSE;
   BEGIN

	v_traffic_ltm := trans_to_ltm(pSrcTable);
	get_range( pSrcTable, v_mindate, v_maxdate, v_minkey, v_maxkey, v_input_row_count);

	v_system_id_str := 'SUBSTR(ltm.ID, 1, INSTR(ltm.ID, ''-'', 1, 1)-1)';
	v_bscdo_id_str := 'SUBSTR(ltm.ID, INSTR(ltm.ID, ''-'', 1, 1)+1, INSTR(ltm.ID, ''-'', 1, 2)-INSTR(ltm.ID, ''-'', 1, 1)-1)';
	v_bscdo_card_id_str := '''' || g_ent_name || ''' || SUBSTR(ltm.ID, INSTR(ltm.ID, ''-'', 1, 2)+1, LENGTH(ltm.ID))';

	v_fullkey_cols_str := v_system_id_str || ' || ''-'' || ' || v_bscdo_id_str || ' || ''-'' || ' || v_bscdo_card_id_str;

	update_wml_bsc_do_card_map( pSrcTable, v_minkey, v_maxkey);

	sqlStr := ' MERGE /*+ USE_HASH(d) INDEX(d) */ INTO mot_tf_do_pm_bsccard_tab D'
			|| ' USING ( SELECT /*+ ORDERED */ DISTINCT('
    			|| v_fullkey_cols_str || ') AS loaderkey, wml.instance_id instance_id,'
			|| ' tstamp, collectionperiod, cpuuserateofapc_avg, cpuuserateofapc_max, memoryuserateofapc_avg, memoryuserateofapc_max, suspectflag'
			|| ' FROM ' || v_traffic_ltm || ' ltm, ' || v_wml_map_tab || ' wml'
			|| ' WHERE wml.system = ' || v_system_id_str || ' AND'
			|| ' wml.bsc_do = ' || v_bscdo_id_str || ' AND'
			|| ' wml.bsc_do_card = ' || v_bscdo_card_id_str || ' AND'
			|| ' wml.loaderkey = ' || v_fullkey_cols_str
			|| ' ) S'
			|| ' ON ( D.id = S.loaderkey  '
			|| ' AND D.tstamp = S.tstamp '
			|| ' AND D.tstamp BETWEEN TO_DATE(:mindate,''MM/DD/YYYY HH24:MI'') AND TO_DATE(:maxdate,''MM/DD/YYYY HH24:MI'') '
			|| ' AND D.id BETWEEN :minkey AND :maxkey)'
      			|| ' WHEN MATCHED THEN '
			|| ' UPDATE SET  '
			|| ' D.instance_id = S.instance_id,'
			|| ' D.collectionperiod = S.collectionperiod,'
			|| ' D.cpuuserate_avg = S.cpuuserateofapc_avg,'
			|| ' D.cpuuserate_max = S.cpuuserateofapc_max,'
			|| ' D.memoryuserate_avg = S.memoryuserateofapc_avg,'
			|| ' D.memoryuserate_max = S.memoryuserateofapc_max,'
			|| ' D.suspectflag = S.suspectflag'
			|| ' WHEN NOT MATCHED THEN '
			|| ' INSERT ( '
			|| ' D.ID, D.tstamp, D.instance_id, '
			|| '  D.collectionperiod, D.cpuuserate_avg, D.cpuuserate_max,'
			|| '  D.memoryuserate_avg, D.memoryuserate_max, D.suspectflag'
			|| ' ) VALUES ( '
			|| ' S.loaderkey,'
			|| ' S.tstamp,'
			|| ' S.instance_id,'
			|| ' S.collectionperiod,'
			|| ' S.cpuuserateofapc_avg,'
			|| ' S.cpuuserateofapc_max,'
			|| ' S.memoryuserateofapc_avg,'
			|| ' S.memoryuserateofapc_max,'
			|| ' S.suspectflag'
			|| ' )';

	WHILE (NOT v_done)
	LOOP
	   BEGIN
		EXECUTE IMMEDIATE sqlStr USING v_mindate, v_maxdate, v_minkey, v_maxkey;
		COMMIT;
		v_done := TRUE;
	   EXCEPTION
		WHEN DUP_VAL_ON_INDEX THEN NULL;
	   END;
	END LOOP;

   END POPULATE_BSCCARD_APC;

-----------------------------------------------------------------------
----------------------- POPULATE_BSCCARD_TF ---------------------------
-----------------------------------------------------------------------
-- This is the main procedure that calls other procedures used to
-- populate BSC_DO_Card traffic table

   PROCEDURE POPULATE_BSCCARD_TF (pSrcTable IN VARCHAR2, pTimeStamp IN VARCHAR2)
   IS
   BEGIN

--	CAGENUMBER_ENT_UPDATE only used in R6.1 in 2.1.x release (temporarily commented out for 2.0.x releases)
--	Procedure needs to be re-added and re-optimized in 2.1.x

	 IF (pSrcTable = 'MOT_TF_DO_PM_CC_TMP') THEN
	    POPULATE_BSCCARD_CC(pSrcTable, pTimeStamp);
		   CAGENUMBER_ENT_UPDATE (pSrcTable);
	 ELSIF (pSrcTable = 'MOT_TF_DO_PM_TC_TMP') THEN
	    POPULATE_BSCCARD_TC(pSrcTable, pTimeStamp);
		   CAGENUMBER_ENT_UPDATE (pSrcTable);
	 ELSIF (pSrcTable = 'MOT_TF_DO_PM_TRA_TMP') THEN
	    POPULATE_BSCCARD_TRA(pSrcTable, pTimeStamp);
		   CAGENUMBER_ENT_UPDATE (pSrcTable);
	 ELSIF (pSrcTable = 'MOT_TF_DO_PM_SVM_TMP') THEN
	    POPULATE_BSCCARD_SVM(pSrcTable, pTimeStamp);
	 ELSIF (pSrcTable = 'MOT_TF_DO_PM_SCA_TMP') THEN
	 	   POPULATE_BSCCARD_SCA(pSrcTable, pTimeStamp);
		   CAGENUMBER_ENT_UPDATE (pSrcTable);
	 ELSIF (pSrcTable = 'MOT_TF_DO_PM_APC_TMP') THEN
	    POPULATE_BSCCARD_APC(pSrcTable, pTimeStamp);
		   CAGENUMBER_ENT_UPDATE (pSrcTable);
	 END IF;

   END POPULATE_BSCCARD_TF;

-----------------------------------------------------------------------------
----------------------------- POPULATE_MCCDO_ID ------------------------------
-----------------------------------------------------------------------------
-- This procedure is the main entry point to this package
-- Invoked by LoadAction.txt

   PROCEDURE Populate_MCCDO_ID   ( pSrcTable IN  VARCHAR2)
   IS
	
	sqlStr		VARCHAR2(4000);
	v_done		BOOLEAN := FALSE;


   BEGIN

	sqlStr := 'DELETE FROM '||pSrcTable||' WHERE btsid is NULL OR mccdoid is NULL';
 
		EXECUTE IMMEDIATE sqlStr;
		COMMIT;


	sqlStr := ' MERGE /*+ USE_HASH(d) INDEX(d) */ INTO '||pSrcTable||' D'
			|| ' USING ( SELECT /*+ ORDERED */ tmptab.TSTAMP as TSTAMP,tmptab.btsid as btsid,tmptab.mccdoid as mccdoid ,tmptab.loaderkey AS loaderkey,'
			|| ' Wm_Generic_Find_Instance.Find_Instance('
			|| ' tmptab.system_id || ''-'' || tmptab.bsc_do_id || ''-'' || tmptab.btsid|| ''-'' || tmptab.mccdoid,''MCC_DO'') AS instance_id'
	      		|| ' FROM ('
			|| '		 SELECT /*+ CARDINALITY(tmp, 1000) */ ID,TSTAMP,'
			|| '		 SUBSTR(tmp.ID, 1, INSTR(tmp.ID, ''-'', 1, 1)-1) || ''-'' || '
			|| '		 SUBSTR(ID, INSTR(ID, ''-'', 1, 1)+1, INSTR(ID, ''-'', 1, 2)-INSTR(ID, ''-'', 1, 1)-1) || ''-'' || tmp.btsid || ''-'' ||tmp.mccdoid'
			|| '		 AS loaderkey,'
			|| '		 SUBSTR(tmp.ID, 1, INSTR(tmp.ID, ''-'', 1, 1)-1) AS system_id,'
			|| '		 SUBSTR(ID, INSTR(ID, ''-'', 1, 1)+1, INSTR(ID, ''-'', 1, 2)-INSTR(ID, ''-'', 1, 1)-1) AS bsc_do_id,'
			|| '		 btsid, mccdoid  '
			|| ' 		 FROM ' || pSrcTable || ' tmp'
			|| '		 where btsid is NOT NULL AND mccdoid is NOT NULL'
  			|| ' ) tmptab'
			||') S'
			|| ' ON (D.TSTAMP = S.TSTAMP AND D.btsid= S.btsid AND D.mccdoid=S.mccdoid ) '
      			|| ' WHEN MATCHED THEN '
			|| ' UPDATE SET  '
			|| ' D.ID = S.loaderkey,'
			|| ' D.instance_id=s.instance_id ';

			--dbms_output.put_line (sqlStr);

		WHILE (NOT v_done)
		LOOP
		   BEGIN
			EXECUTE IMMEDIATE sqlStr;
			COMMIT;
		v_done := TRUE;
		   EXCEPTION
			WHEN DUP_VAL_ON_INDEX THEN NULL;
			WHEN OTHERS THEN
				RAISE_APPLICATION_ERROR (-20090, 'Something wrong with aggregate data from ' || pSrcTable || ' - ' || SQLERRM);
		   END;
		END LOOP;

   END Populate_MCCDO_ID;

-----------------------------------------------------------------------
----------------------------- POPULATE_TF -----------------------------
-----------------------------------------------------------------------
-- This procedure is the main entry point to this package
-- Invoked by LoadAction.txt

   PROCEDURE Populate_TF   ( pSrcTable IN  VARCHAR2, pDestTable IN VARCHAR2, pIntermTable IN VARCHAR2, pLoadResult IN VARCHAR2 )
   IS
	v_min            DATE;
	v_max            DATE;
	cntGetData       INTEGER;
	cntBuildStr      INTEGER;
	sqlStr           VARCHAR2(4000);
	is_bsccard_tab   INTEGER := 0;

       -- use index by table to fetch info
       TYPE             colTab  IS TABLE OF WMD_COL_DEF.col_name%TYPE
                           INDEX BY BINARY_INTEGER;
       retTabCol        colTab;

       TYPE             aggrTab IS TABLE OF wmd_field_def.field_aggregator%TYPE
                           INDEX BY BINARY_INTEGER;
       retTabAggr       aggrTab;

       -- user-defined exception
       noDataDictInfo   EXCEPTION;
       noHandleAggrType EXCEPTION;

   BEGIN

	IF pLoadResult = 'loadOK'
	THEN
	   -- is_bsccard_tab = 1 means that the particular TMP table will be used to populate BSC_DO_Card entity
	   -- nodekind_ENT_UPDATE only done for R6.1 data, cardkind_ENT_UPDATE introduced in R6

	   IF (pSrcTable = 'MOT_TF_DO_PM_APC_TMP') OR (pSrcTable = 'MOT_TF_DO_PM_APC4_TMP') OR
		(pSrcTable = 'MOT_TF_DO_PM_SCA_TMP') OR (pSrcTable = 'MOT_TF_DO_PM_TC_TMP') OR
		(pSrcTable = 'MOT_TF_DO_PM_CC_TMP') OR (pSrcTable = 'MOT_TF_DO_PM_TRA_TMP') OR
		(pSrcTable = 'MOT_TF_DO_PM_SVM_TMP') OR (pSrcTable = 'MOT_TF_DO_PM_BSC_TMP')
	   THEN
		get_entity_info(pSrcTable);

		-- There's no cardkind field in SCA entity as it only introduced in R6.1
		IF (pSrcTable = 'MOT_TF_DO_PM_SCA_TMP' OR pSrcTable = 'MOT_TF_DO_PM_APC_TMP' OR pSrcTable = 'MOT_TF_DO_PM_BSC_TMP') THEN
		   is_bsccard_tab := 1;
		ELSE
		   -- MCR015 - no longer loading CardKind data into entity tables
		   --cardkind_ENT_UPDATE ( pSrcTable );
		   is_bsccard_tab := 1;
		END IF;

--		   Next few lines only used in R6.1 in 2.1.x release (temporarily commented out for 2.0.x releases)
		   IF (pSrcTable <> 'MOT_TF_DO_PM_SVM_TMP' AND pSrcTable <> 'MOT_TF_DO_PM_APC4_TMP') THEN
		   	  nodekind_ENT_UPDATE ( pSrcTable );
		   END IF;

	   END IF;

	   -- save table column and field aggregator pair information to tables
	   SELECT /*+ USE_HASH (wmd_field_def) */  cd.col_name, fd.field_aggregator BULK COLLECT INTO retTabCol, retTabAggr FROM WMD_FIELD_DEF fd, WMD_COL_DEF cd
	   WHERE fd.field_id = cd.field_id AND cd.table_id IN
	   (SELECT /*+ FIRST_ROWS*/ table_id FROM WMD_DICT_TABLE WHERE table_name = UPPER(pDestTable));

	   -- continue if the traffic table information in data dictionary
           IF (SQL%ROWCOUNT > 1) THEN
		-- build sql statement to aggregate data

		sqlStr := 'INSERT INTO '
			|| pIntermTable
			|| ' (ID, tstamp, instance_id';

	   FOR cntBuildStr IN retTabCol.first..retTabCol.last LOOP
		sqlStr := sqlStr
			|| ', '
			|| retTabCol(cntBuildStr);
           END LOOP;

           -- Logic to effectively handle the situation where data is only available
           -- for one of the two 5 min intervals.
           IF (TO_CHAR(v_min, 'MM-DD-YYYY HH24:MI:SS') = TO_CHAR(v_max, 'MM-DD-YYYY HH24:MI:SS')) AND
              (SUBSTR(TO_CHAR(v_min, 'MM-DD-YYYY HH24:MI:SS'), 16, 1) = 5)
           THEN
             sqlStr := sqlStr
                     || ') SELECT ID, (min(tstamp)-5/1440), instance_id';
           ELSE
             sqlStr := sqlStr
                     || ') SELECT ID, min(tstamp), instance_id';
           END IF;

	   FOR cntBuildStr IN retTabCol.first..retTabCol.last LOOP
             IF (retTabAggr(cntBuildStr) = 'S') THEN
               sqlStr := sqlStr || ', sum(' || retTabCol(cntBuildStr) || ')';
             ELSIF (retTabAggr(cntBuildStr) = 'A') THEN
               sqlStr := sqlStr || ', avg(' || retTabCol(cntBuildStr) || ')';
             ELSIF (retTabAggr(cntBuildStr) = 'Q') THEN
               sqlStr := sqlStr || ', max(' || retTabCol(cntBuildStr) || ')';
             ELSIF (retTabAggr(cntBuildStr) = 'M') THEN
               sqlStr := sqlStr || ', min(' || retTabCol(cntBuildStr) || ')';
             ELSIF (retTabAggr(cntBuildStr) = 'L') THEN
               sqlStr := sqlStr || ', max(' || retTabCol(cntBuildStr) || ')';
             ELSIF (retTabAggr(cntBuildStr) = 'C') THEN
               sqlStr := sqlStr || ', max(' || retTabCol(cntBuildStr) || ')';
             ELSIF (retTabAggr(cntBuildStr) = 'd') THEN
               sqlStr := sqlStr || ', stddev(' || retTabCol(cntBuildStr) || ')';
             ELSIF (retTabAggr(cntBuildStr) = 'J') THEN
               sqlStr := sqlStr || ', sum(' || retTabCol(cntBuildStr) || ')';
             ELSIF (retTabAggr(cntBuildStr) = 'F') THEN
               sqlStr := sqlStr || ', avg(' || retTabCol(cntBuildStr) || ')';
             ELSIF (retTabAggr(cntBuildStr) = 'I') THEN
               sqlStr := sqlStr || ', sum(' || retTabCol(cntBuildStr) || ')';
	     ELSIF (retTabAggr(cntBuildStr) = 'T') THEN
               sqlStr := sqlStr || ', avg(' || retTabCol(cntBuildStr) || ')';
             ELSIF (retTabAggr(cntBuildStr) = 'N') THEN
               sqlStr := sqlStr || ', NULL';
             -- raise application error
             ELSE
               RAISE noHandleAggrType;
             END IF;
           END LOOP;

           sqlStr := sqlStr
                     || ' FROM '
                     || pSrcTable
                     || ' GROUP BY ID, instance_id';

           EXECUTE IMMEDIATE sqlStr;

         -- raise application error
         ELSE
           RAISE noDataDictInfo;
         END IF;

	-- Run the POPULATE_BSCSCARD_TF when is_bsccard_tab=1 to populate the mot_tf_do_pm_bsccard_tab (BSC_DO_Card entity) table
       IF (is_bsccard_tab = 1)
         THEN
           IF (TO_CHAR(v_min, 'MM-DD-YYYY HH24:MI:SS') = TO_CHAR(v_max, 'MM-DD-YYYY HH24:MI:SS')) AND
              (SUBSTR(TO_CHAR(v_min, 'MM-DD-YYYY HH24:MI:SS'), 16, 1) = 5)
           THEN
	 	   	  POPULATE_BSCCARD_TF(pSrcTable, TO_CHAR((v_min-5/1440), 'YYYY/MM/DD HH24:MI:SS'));
           ELSE
	       	  POPULATE_BSCCARD_TF(pSrcTable, TO_CHAR(v_min, 'YYYY/MM/DD HH24:MI:SS'));
           END IF;
         END IF;

       END IF;

     EXCEPTION
       WHEN noDataDictInfo THEN
         RAISE_APPLICATION_ERROR (-20070, 'No dictionary field found for table ' || pDestTable);
       WHEN noHandleAggrType THEN
         RAISE_APPLICATION_ERROR (-20080, 'Aggregation type is not handled for table ' || pDestTable);
       WHEN OTHERS THEN
         RAISE_APPLICATION_ERROR (-20090, 'Something wrong with aggregate data from ' || pSrcTable || ' to ' || pDestTable || ' - ' || SQLERRM);

   END Populate_TF;

   --MCR 013
-- Fix for APAR raised against RP4.0 where we were seeing  ORA-30926: unable to get a stable set of rows in the source tables                                        
PROCEDURE Populate_FDN ( pSrcTable IN  VARCHAR2 )
   IS
   sqlStr			VARCHAR2(32000);
   v_system_id_str		VARCHAR2(500);
   v_mccdo_id_str		VARCHAR2(500);
   v_mccdo_modem_id_str	VARCHAR2(2500);
   v_fullkey_cols_str	VARCHAR2(5000);
   v_mindate 		VARCHAR2(20);
   v_maxdate 		VARCHAR2(20);
   v_minkey 		VARCHAR2(256);
   v_maxkey 		VARCHAR2(256);
   v_input_row_count 	INTEGER := 0;
   v_traffic_ltm		VARCHAR2(30);
   v_done			BOOLEAN := FALSE;
   
   v_banner	  VARCHAR2(255);
   CURSOR c_mccmdm IS SELECT mccmdm.id,apcmdm3.ID as APCMDMFDN, to_char(apcmdm3.TSTAMP, 'MM/DD/YYYY HH24:MI:SS') as TSTAMP
   		  		   	  FROM MOT_TF_DO_PM_MCCMDM_TMP mccmdm,MOT_TF_DO_PM_APCMDM_TMP apcmdm,
					  MOT_TF_DO_PM_APCMDM3_TMP apcmdm3 WHERE  apcmdm.ID = apcmdm3.ID AND apcmdm.TSTAMP= apcmdm3.TSTAMP
					  AND apcmdm.tstamp = mccmdm.tstamp
					  AND apcmdm.mccdoid = SUBSTR(mccmdm.ID, INSTR(mccmdm.ID, '-', 1,3)+1, INSTR(mccmdm.ID, '-', 1, 4)-INSTR(mccmdm.ID, '-', 1, 3)-1)
					  AND apcmdm3.CHANNELNUMBER = mccmdm.CHANNELNUMBER AND
					  apcmdm3.SECTORNUMBER = SUBSTR(mccmdm.ID,instr(mccmdm.ID, '-', 1, 4)+1);
					  
   CURSOR c_apcmdm IS SELECT mccmdm.id as MCCMDMFDN,apcmdm3.ID, to_char(apcmdm3.TSTAMP, 'MM/DD/YYYY HH24:MI:SS') as TSTAMP
 		  		   	  FROM MOT_TF_DO_PM_MCCMDM_TMP mccmdm,MOT_TF_DO_PM_APCMDM_TMP apcmdm,
					  MOT_TF_DO_PM_APCMDM3_TMP apcmdm3 WHERE  apcmdm.ID = apcmdm3.ID AND apcmdm.TSTAMP= apcmdm3.TSTAMP
					  AND apcmdm.tstamp = mccmdm.tstamp
					  AND apcmdm.mccdoid = SUBSTR(mccmdm.ID, INSTR(mccmdm.ID, '-', 1,3)+1, INSTR(mccmdm.ID, '-', 1, 4)-INSTR(mccmdm.ID, '-', 1, 3)-1)
					  AND apcmdm3.CHANNELNUMBER = mccmdm.CHANNELNUMBER AND
					  apcmdm3.SECTORNUMBER = SUBSTR(mccmdm.ID,instr(mccmdm.ID, '-', 1, 4)+1);					  
					  
      BEGIN
	select banner INTO v_banner from v$version where banner like 'Oracle%';
	  
	v_traffic_ltm := trans_to_ltm(pSrcTable);
   	
	get_entity_info(pSrcTable);
	get_range( pSrcTable, v_mindate, v_maxdate, v_minkey, v_maxkey, v_input_row_count);

	IF pSrcTable = 'MOT_TF_DO_PM_MCCMDM_TMP' THEN
	
	 sqlStr := 'MERGE /*+ USE_HASH(d) INDEX(d) */ INTO MOT_TF_DO_PM_MCCMDM_TMP D' 
 		||' USING (SELECT /*+ ORDERED */ mccmdm.id,apcmdm3.ID as APCMDMFDN,apcmdm3.TSTAMP'
 		||' FROM MOT_TF_DO_PM_MCCMDM_TMP mccmdm,MOT_TF_DO_PM_APCMDM_TMP apcmdm,'
 		||' MOT_TF_DO_PM_APCMDM3_TMP apcmdm3 WHERE  apcmdm.ID = apcmdm3.ID AND apcmdm.TSTAMP= apcmdm3.TSTAMP' 
 		||' AND apcmdm.tstamp = mccmdm.tstamp'
 		||' AND apcmdm.mccdoid = SUBSTR(mccmdm.ID, INSTR(mccmdm.ID, ''-'', 1,3)+1, INSTR(mccmdm.ID, ''-'', 1, 4)-INSTR(mccmdm.ID, ''-'', 1, 3)-1)' 
 		||' AND apcmdm3.CHANNELNUMBER = mccmdm.CHANNELNUMBER AND' 
		||' apcmdm3.SECTORNUMBER = SUBSTR(mccmdm.ID,instr(mccmdm.ID, ''-'', 1, 4)+1)) S' 
 		||' ON (D.id = S.id AND'
 		||' D.tstamp=S.tstamp' 
		||' )' 
		||' WHEN MATCHED THEN  UPDATE SET   D.APCMODEMFDN=S.APCMDMFDN';
        ELSIF  pSrcTable = 'MOT_TF_DO_PM_APCMDM3_TMP' THEN 
	 sqlStr := 'MERGE /*+ USE_HASH(d) INDEX(d) */ INTO MOT_TF_DO_PM_APCMDM3_TMP D' 
 		||' USING (SELECT /*+ ORDERED */ mccmdm.id as MCCMDMFDN,apcmdm3.ID,apcmdm3.TSTAMP'
 		||' FROM MOT_TF_DO_PM_MCCMDM_TMP mccmdm,MOT_TF_DO_PM_APCMDM_TMP apcmdm,'
 		||' MOT_TF_DO_PM_APCMDM3_TMP apcmdm3 WHERE  apcmdm.ID = apcmdm3.ID AND apcmdm.TSTAMP= apcmdm3.TSTAMP' 
 		||' AND apcmdm.tstamp = mccmdm.tstamp'
 		||' AND apcmdm.mccdoid = SUBSTR(mccmdm.ID, INSTR(mccmdm.ID, ''-'', 1,3)+1, INSTR(mccmdm.ID, ''-'', 1, 4)-INSTR(mccmdm.ID, ''-'', 1, 3)-1)' 
 		||' AND apcmdm3.CHANNELNUMBER = mccmdm.CHANNELNUMBER AND' 
		||' apcmdm3.SECTORNUMBER = SUBSTR(mccmdm.ID,instr(mccmdm.ID, ''-'', 1, 4)+1)) S' 
 		||' ON (D.id = S.id AND'
 		||' D.tstamp=S.tstamp' 
		||' )' 
		||' WHEN MATCHED THEN  UPDATE SET   D.MCCMODEMFDN=S.MCCMDMFDN';
	 END IF;	

	 
		WHILE (NOT v_done)
		LOOP
	   		BEGIN
	   			 EXECUTE IMMEDIATE sqlStr;
	   			 COMMIT;
	   			 v_done := TRUE;
	                EXCEPTION
	   	     	    WHEN DUP_VAL_ON_INDEX THEN NULL;	 
			END;
                END LOOP;
EXCEPTION WHEN OTHERS THEN
          IF SQLCODE = '-30926' THEN
              dbms_output.put_line('exception block: -30926');
	      DECLARE 
	   		v_apcmdm_fdn VARCHAR2(255);
	   		CURSOR mccmdmfdn_cur IS SELECT id, tstamp, channelnumber
	   		FROM MOT_TF_DO_PM_MCCMDM_TMP;
	   		v_mccmdm_fdn VARCHAR2(255);
	   		CURSOR apcmdmfdn_cur IS SELECT apcmdm3.ID,apcmdm3.TSTAMP, apcmdm.mccdoid, apcmdm3.CHANNELNUMBER, apcmdm3.SECTORNUMBER
	   		FROM MOT_TF_DO_PM_APCMDM_TMP apcmdm, MOT_TF_DO_PM_APCMDM3_TMP apcmdm3 
			WHERE  apcmdm.ID = apcmdm3.ID AND apcmdm.TSTAMP= apcmdm3.TSTAMP;
	   
	       BEGIN
	            IF pSrcTable = 'MOT_TF_DO_PM_APCMDM3_TMP' THEN
	               BEGIN
	   	      	     FOR apcmdmfdn_rec IN apcmdmfdn_cur
		             LOOP
			        v_mccmdm_fdn := NULL;
				BEGIN
					SELECT mccmdm.ID INTO v_mccmdm_fdn FROM MOT_TF_DO_PM_MCCMDM_TMP mccmdm
					WHERE mccmdm.tstamp = apcmdmfdn_rec.tstamp
					AND SUBSTR(mccmdm.ID, INSTR(mccmdm.ID, '-', 1,3)+1, INSTR(mccmdm.ID, '-', 1, 4)-INSTR(mccmdm.ID, '-', 1, 3)-1)= apcmdmfdn_rec.mccdoid 
					AND mccmdm.CHANNELNUMBER = apcmdmfdn_rec.channelnumber 
					AND SUBSTR(mccmdm.ID,INSTR(mccmdm.ID, '-', 1, 4)+1) = apcmdmfdn_rec.sectornumber;

					UPDATE MOT_TF_DO_PM_APCMDM3_TMP SET MCCMODEMFDN=v_mccmdm_fdn WHERE id=apcmdmfdn_rec.id AND tstamp=apcmdmfdn_rec.tstamp;
				EXCEPTION 
				WHEN TOO_MANY_ROWS THEN
				    NULL;
				WHEN NO_DATA_FOUND THEN
					 NULL;	
				END;	
			    END LOOP;
			    CLOSE apcmdmfdn_cur;
			
	              EXCEPTION 
	  		WHEN OTHERS THEN
	  	     	 NULL;
	  	      END;		
		
	      ELSIF pSrcTable = 'MOT_TF_DO_PM_MCCMDM_TMP' THEN
	         BEGIN
	   		FOR mccmdmfdn_rec IN mccmdmfdn_cur		
			LOOP
			        v_apcmdm_fdn := NULL;
				BEGIN
					SELECT apcmdm.id INTO v_apcmdm_fdn FROM MOT_TF_DO_PM_APCMDM_TMP apcmdm, MOT_TF_DO_PM_APCMDM3_TMP apcmdm3
					WHERE apcmdm.id = apcmdm3.id AND apcmdm.tstamp = apcmdm3.tstamp 
					AND apcmdm.tstamp = mccmdmfdn_rec.tstamp
					AND apcmdm.mccdoid = SUBSTR(mccmdmfdn_rec.id, INSTR(mccmdmfdn_rec.id, '-', 1,3)+1, INSTR(mccmdmfdn_rec.id, '-', 1, 4)-INSTR(mccmdmfdn_rec.id, '-', 1, 3)-1) 
					AND apcmdm3.channelnumber = mccmdmfdn_rec.channelnumber 
					AND apcmdm3.sectornumber = SUBSTR(mccmdmfdn_rec.id,INSTR(mccmdmfdn_rec.id, '-', 1, 4)+1);	

					UPDATE MOT_TF_DO_PM_MCCMDM_TMP SET APCMODEMFDN=v_apcmdm_fdn WHERE id=mccmdmfdn_rec.id AND tstamp=mccmdmfdn_rec.tstamp;
				EXCEPTION 
				WHEN TOO_MANY_ROWS THEN
				    NULL;
				WHEN NO_DATA_FOUND THEN
					 NULL;	
				END;
			END LOOP;
			CLOSE mccmdmfdn_cur;
			
	        EXCEPTION 
	  		WHEN OTHERS THEN
	  	    	 NULL;
	       END;
	  END IF;  -- if pSrcTable == 
    END; -- BEGIN for  if sqlCode = -309

		
ELSE -- if sqlCode =  -309
	 
       RAISE_APPLICATION_ERROR (-20090, 'Something wrong with aggregate data for ' || ' ' || pSrcTable || ' - ' || SQLERRM);

END IF;

END Populate_FDN;


   -- SEAde66187: Calculate value for intermediate peg MFPAUsersPercentage_Int (BSCDO entity) during load time
   FUNCTION calc_MFPAUsersPercentage_Int(pID IN VARCHAR2, pTimeStamp IN VARCHAR2, pPresentSessions_AVG IN FLOAT) RETURN FLOAT
   IS
    v_TotalMFPAUsers FLOAT;
   BEGIN
    SELECT SUM(TotalMFPAUsers) INTO v_TotalMFPAUsers 
	FROM MOT_TF_DO_PM_CC_TMP WHERE ID LIKE pID || '-%' AND TSTAMP = TO_DATE(pTimeStamp,'MM/DD/YYYY HH24:MI');
   	RETURN 100 *  (v_TotalMFPAUsers / pPresentSessions_AVG );

   EXCEPTION WHEN ZERO_DIVIDE THEN
        RETURN NULL;
   END calc_MFPAUsersPercentage_Int;

END Moto_Evdo_Load;
/
